Package: dictionar6
Title: R6 Dictionary Interface
Description: Efficient object-oriented R6 dictionary capable of holding objects of any class, including R6. Typed and untyped dictionaries are provided as well as the 'usual' dictionary methods that are available in other OOP languages, for example listing keys, items, values, and methods to get/set these.
Version: 0.1.3
Authors@R:
  person(given = "Raphael",
             family = "Sonabend",
             role = c("aut","cre"),
             email = "raphaelsonabend@gmail.com",
             comment = c(ORCID = "0000-0001-9225-4654"))
License: MIT + file LICENSE
URL: https://xoopR.github.io/dictionar6/, https://github.com/xoopR/dictionar6/
BugReports: https://github.com/xoopR/dictionar6/issues
Config/testthat/edition: 3
Encoding: UTF-8
NeedsCompilation: no
Roxygen: list(markdown = TRUE, r6 = TRUE)
RoxygenNote: 7.1.1
Imports:
  ooplah,
  R6
Suggests:
  testthat
