## Test Results

No NOTEs, WARNINGs, or ERRORs.

## Test environments

* Windows, R Release
* Mac OS X, R Release
* Ubuntu, R Release
* Ubuntu, R Oldrel
* Ubuntu, R Devel

## Reverse Dependencies

All reverse dependencies OK.